package de.hsbo.fbv.bmg.geometry.mysimple;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;

/**
 * Abstrakte Elternklasse der einfachen Geometrieobjekte.<br>
 * Geometry enthlt bereits das Attribut path, da alle Geometrien als 
 * Path-Objekte darstellbar sind.
 * @author A.Zimmermann, HS Bochum, 2011
 * @see BMG, Kap. 4.1
 * 
 */
public abstract class SimpleGeometry implements Shape, Serializable {

	/** Konstante: Versionsnummer */
	private static final long serialVersionUID = 1L;
	
	/** Klassenattribut: Toleranz EPS, kleinste auflsbare Koordinaten- und Lngengre */ 
	public static double EPS = 1.e-3;

	/**
	 * Attribut path: alle Geometrien sind als Path2D-Objekte darstellbar.
	 * Das Attribut erfllt zugleich alle Methoden der Schnittstelle Shape 
	 * (Delegationsprizip). Die Aufgabe der konkreten Kindklassen besteht 
	 * darin, fr das Attribut path ein passendes Path2D-Objekt zu bilden.
	 */
	Path2D path = null;

	/**
	 * Abstrakte Methode, gibt die Anzahl der Elemente an.
	 * Diese Methode wird in den Kindklassen unterschiedlich interpretiert.
	 * @return 
	 */
	public abstract int size();

	/**
	 * Abstrakte Methode: gibt die Elemente als Text aus.
	 * @return
	 */
	public abstract String toText();

	/**
	 * Abstrakte Methode: zwingt die konkreten Kindklassen, fr ihr path-Attribut
	 * ein passendes Path2D-Objekt zu generieren. Verwendung innerhalb der 
	 * Konstruktoren.
	 */
	protected abstract void setPath();

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public Rectangle getBounds() {
		return path.getBounds();
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public Rectangle2D getBounds2D() {
		return path.getBounds2D();
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean contains(double x, double y) {
		return path.contains(x, y);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean contains(Point2D p) {
		return path.contains(p);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean intersects(double x, double y, double w, double h) {
		return path.intersects(x, y, w, h);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean intersects(Rectangle2D r) {
		return path.intersects(r);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean contains(double x, double y, double w, double h) {
		return path.contains(x, y, w, h);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public boolean contains(Rectangle2D r) {
		return path.contains(r);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public PathIterator getPathIterator(AffineTransform at) {
		return path.getPathIterator(at);
	}

	/**
	 * Delegat-Methode von Path2D fr Shape.
	 */
	public PathIterator getPathIterator(AffineTransform at, double flatness) {
		return path.getPathIterator(at, flatness);
	}
	


}
