package de.hsbo.fbv.bmg.geometry.mysimple;

import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.io.Serializable;

/**
 * Klasse SimplePoint zur Speicherung punktfrmiger Objekte.
 * Konkrete Kindklasse von SimpleGeometry, die Punkte im R oder R speichert.
 * @author A.Zimmermann, HS Bochum, 2011
 * @see BMG, Kap. 4.1
 * 
 */
public class SimplePoint extends SimpleGeometry implements Comparable<SimplePoint>, Serializable {
	
	/** Konstante: Versionsnummer */
	private static final long serialVersionUID = 1L;
	/** Konstante: Index der Komponente x */
	public static final int X = 0;
	/** Konstante: Index der Komponente y */
	public static final int Y = 1;
	/** Konstante: Index der Komponente z */
	public static final int Z = 2;
	
	/** Klassenattribut: enthlt die Sortierrichtung von Punkten */
	private static int compareIndex = X;
	
	/** Attribut: double-Feld mit drei Elementen */
	double[] coords = {Double.NaN, Double.NaN, Double.NaN};
	
	/**
	 * Konstruktor aus Point2D-Objekt
	 * @param point 
	 */
	public SimplePoint(Point2D point) {
		this.coords[X] = point.getX();
		this.coords[Y] = point.getY();
		this.setPath();
	}
	
	/**
	 * Konstruktor aus x- und y-Wert
	 * @param x
	 * @param y
	 */
	public SimplePoint(double x, double y) {
		this.coords[X] = x;
		this.coords[Y] = y;
		this.setPath();
	}

	/**
	 * Konstruktor aus x-, y- und z-Wert
	 * @param x
	 * @param y
	 * @param z
	 */
	public SimplePoint(double x, double y, double z) {
		this.coords[X] = x;
		this.coords[Y] = y;
		this.coords[Z] = z;
		this.setPath();
	}
	
	/**
	 * Konstruktor aus SimplePoint-Objekt (Kopier-Konstruktor) 
	 * @param p
	 */
	public SimplePoint(SimplePoint p) {
		this.coords[X] = p.coords[X];
		this.coords[Y] = p.coords[Y];
		this.coords[Z] = p.coords[Z];
		this.setPath();		
	}
	
	/**
	 * Klassenmethode: fragt die Sortierrichtung ab.
	 * @return
	 */
	public static int getCompareIndex() {
		return compareIndex;
	}

	/**
	 * Klassenmethode: legt die Sortierrichtung fest.
	 * @param compareIndex
	 */
	public static void setCompareIndex(int compareIndex) {
		SimplePoint.compareIndex = compareIndex;
	}
	
	/**
	 * Gibt den Wert der Komponente i zurck.
	 * @param i
	 * @return
	 */
	public double get(int i) {
		return this.coords[i];
	}

	/**
	 * Override-Methode von Object
	 */
	public String toString() {
		if(coords[Z]==Double.NaN)
			return coords[X]+" "+coords[Y];
		else
			return coords[X]+" "+coords[Y]+" "+coords[Z];
	}
	
	/**
	 * Override-Methode von SimpleGeometry
	 */
	public String toText() {
		return "POINT("+this.toString()+")";
	}
	
	/**
	 * Wandelt das Objekt in ein Point2D-Objekt.
	 * @return
	 */
	public Point2D toPoint2D() {
		return new Point2D.Double(this.coords[X], this.coords[Y]);
	}
	
	/**
	 * Erzeugt aus der x- und der y-Koordinate ein neues SimplePoint-Objekt.
	 * Aus einem 3D-Punkt entsteht so die Projektion in die XY-Ebene. 
	 * @return
	 */
	public SimplePoint toPointXY() {
		return new SimplePoint(coords[X], coords[Y]);
	}
	
	/**
	 * Erzeugt aus dem gegebenen Punkt und einer z-Koordinate einen 3D-Punkt.
	 * @param z
	 * @return
	 */
	public SimplePoint toPointXYZ(double z) {
		return new SimplePoint(coords[X], coords[Y], z);
	}
	
	/**
	 * Setzt die Z-Komponente.
	 * @param z
	 */
	public void setZ(double z) {
		this.coords[Z] = z;
		this.setPath();
	}

	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Erzeugt ein Graphik-Objekt zur Darstellung in einem Anzeigefenster.
	 * Es entsteht eine Strecke mit lagegleichem Anfangs- und Endpunkt, die je nach 
	 * Strichmodus (BasicStroke) als Kreis (CAP_ROUND), als Quadrat (CAP_SQARE) 
	 * oder gar nicht (CAP_BUTT) erscheint. 
	 */
	public void setPath() {
		this.path = new Path2D.Double();
		path.moveTo(coords[X], coords[Y]);
		path.lineTo(coords[X], coords[Y]);
	}
	
	/**
	 * Override-Methode von SimpleGeometry:<br>
	 * Punkte geben die Anzahl der Komponenten (2 fr 2D, 3 fr 3D) zurck.
	 */
	public int size() {
		if(this.coords[Z]!=Double.NaN)
			return 3;
		else
			return 2;
	}

	/**
	 * Override-Methode von Comparable:<br>
	 * In Abhngigkeit von der voreingestellten Sortierrichtung compareIndex
	 * wird die relative Rangordnung zu einem weiteren Punkt p bestimmt. 
	 * @return -1, 0, +1
	 */
	public int compareTo(SimplePoint p) {
		int c = getCompareIndex();
		if(this.coords[c]<p.coords[c])
			return -1;
		else if(this.coords[c]==p.coords[c])
			return 0;
		else // if(this.point[c]>p.point[c])
			return 1;
	}

	/**
	 * Vergleicht die Punkte this und p auf Lagegleichheit in der XY-Ebene.
	 * Vergleichgrundlage ist das Toleranzma EPS von SimpleGeometry.
	 * @param p
	 * @return true, falls die Koordinaten-Differenzen kleiner als EPS sind, sonst false
	 */
	public boolean equalsXY(SimplePoint p) {
		if(Math.abs(coords[X]-p.coords[X])<EPS && 
				Math.abs(coords[Y]-p.coords[Y])<EPS)
			return true;
		else 
			return false;		
	}

	/**
	 * Vergleicht die Punkte this und p auf gleiche Lage in Z.
	 * Vergleichgrundlage ist das Toleranzma EPS von SimpleGeometry.
	 * @param p
	 * @return true, falls beide Punkte ber gltige Z-Koordinaten verfgen und 
	 *         die Koordinaten-Differenz kleiner als EPS ist, sonst false
	 */	
	public boolean equalsZ(SimplePoint p) {
		if(!Double.isNaN(this.coords[Z]) 
				&& !Double.isNaN(p.coords[Z])
				&& Math.abs(coords[Y]-p.coords[Y])<EPS)
			return true;
		else 
			return false;	
	}
	
	/**
	 * Vergleicht die Punkte this und p auf gleiche Lage.
	 * Vergleichgrundlage ist das Toleranzma EPS von SimpleGeometry.
	 * @param p
	 * @return true, falls beide Punkte gleiche Koordinaten-Dimension haben und 
	 *         alle Koordinaten-Differenzen kleiner als EPS sind, sonst false
	 */
	public boolean equals(SimplePoint p) {
		if(this.size()!=p.size())
			return false;
		else if(this.size()==2 && this.equalsXY(p))
			return true;
		else if(this.size()==3 && this.equalsXY(p) && this.equalsZ(p))
			return true;
		else
			return false;
	}
}
